import os
from pickle import NONE
import string
import requests  # 导入requests库，用于发送HTTP请求
import json  # 导入json库，用于处理JSON数据
import time  # 导入time库，用于处理时间相关功能


def is_string_empty(value):
    if value == None:
        return True
    """
    判断给定的字符串是否为空。
    :param value: 要判断的字符串，可以为 None 或 空字符串
    :return: 如果是 None 或 空字符串，返回 True；否则返回 False。
    """
    return str(value) is None or str(value).strip() == ""


# 创建一个私人助手的类
class MyAssitantAi:

    MINMAX_HOST = "https://api.minimax.chat/v1"

    def __init__(self, group_id, api_key, file_path, prompt):
        """
        构造函数。

        参数:
        group_id (str): 在minmax官网注册后获得的用于API身份认证的组ID。
        api_key (str): 在minmax官网注册后获得的用于API身份认证的API密钥。
        file_path (str): 提供的外部知识库的文件路径。
        prompt (str): 提出的问题。

        """
        self.group_id = group_id
        self.api_key = api_key
        self.file_path = file_path
        self.prompt = prompt

    # 用于上传时的身份校验
    def header(self):
        return {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }

    # 同样检索的时候身份校验。
    def headers_retrieval(self):
        return {
            "Authorization": f"Bearer {self.api_key}",
            "authority": "api.minimax.chat",
        }

    # create_file 函数的目的是将一个文件上传到 Minimax API

    def create_file(self):
        # 定义文件上传的 API 的 URL 地址。
        # GroupId 是用于指定文件上传目标组的查询参数。
        url = f"{self.MINMAX_HOST}/files/upload?GroupId={self.group_id}"

        # 定义要发送的数据。'purpose' 字段用于说明上传文件的目的，这里是用于助手功能。
        data = {"purpose": "assistants"}

        # 打开要上传的文件，使用二进制模式 ('rb') 读取文件内容。'file' 字段用于指定上传的文件。
        files = {"file": open(r"{}".format(self.file_path), "rb")}

        # 发送 POST 请求到 API 端点，包含 headers、data 和 files。
        # headers_retrieval 是定义请求头，data 包含上传文件的目的，files 包含实际要上传的文件。
        response = requests.post(
            url, headers=self.headers_retrieval(), data=data, files=files
        )

        # 将 API 响应的 JSON 数据解析并返回。response.json() 方法将响应内容转换为 JSON 格式。
        return response.json()

    def create_assistant(self, file_id):
        # 构建 API 请求的 URL，使用 GroupId 作为查询参数
        url = f"{self.MINMAX_HOST}/assistants/create?GroupId={self.group_id}"

        # 准备请求的负载数据，将数据转换为 JSON 格式的字符串
        payload = json.dumps(
            {
                "model": "abab6.5s-chat",  # 指定要使用的模型版本，这里是 'abab6.5s-chat'
                "name": "显卡购买参考助手",  # 助手的名称
                "description": "在用户给出购买显卡预算，并给出显卡性能指标要求的前提下，帮用户筛选出价格合适，性能满足的显卡型号",  # 对助手的描述，说明其功能和用途
                "instructions": "擅长帮用户在价格和显卡性能之间作出权衡，帮用户做出最佳的购买建议，并说明详细的建议理由",  # 助手的设定或指令，说明助手的工作原理和目标
                "file_ids": [  # 列表包含了与助手相关的文件 ID
                    str(file_id)  # 将传入的文件 ID 转换为字符串形式
                ],
                "tools": [  # 列表包含助手使用的工具
                    {
                        "type": "retrieval"
                    }  # 使用 'retrieval' 工具，可能用于从文件中检索数据
                ],
            }
        )
        # 发送 POST 请求到 API，创建助手
        response = requests.post(url, headers=self.header(), data=payload)
        # 返回 API 的响应，解析为 JSON 格式
        return response.json()

    # 创建线程的函数
    def create_thread(self):
        # 构造请求的 URL，包含 GroupId 参数
        url = f"{self.MINMAX_HOST}/threads/create?GroupId={self.group_id}"

        # 发送 POST 请求以创建一个新线程
        response = requests.post(url, headers=self.header())

        # 返回响应的 JSON 数据
        return response.json()

        # 往指定线程中添加消息的函数

    def add_message_to_thread(self, thread_id, prompt):
        # 构造请求的 URL，包含 GroupId 参数
        url = f"{self.MINMAX_HOST}/threads/messages/add?GroupId={self.group_id}"

        # 构造要发送的消息数据
        payload = json.dumps(
            {
                "thread_id": thread_id,  # 指定要向哪个线程添加消息
                "role": "user",  # 消息发送者的角色，这里是 'user'
                "content": prompt,  # 消息内容
            }
        )

        # 发送 POST 请求以向指定线程添加消息
        response = requests.post(url, headers=self.header(), data=payload)

        # 返回响应的 JSON 数据
        return response.json()

    # 运行助手

    def run_thread_with_assistant(self, thread_id, assistant_id):
        # 这里可以选择添加延时（time.sleep(20)）以等待助手的创建和初始化完成。注释掉的代码是为了处理助手创建后可能需要的时间，
        # 用于助手的数据向量化和存储。在实际应用中，可以通过检索助手状态来判断助手是否已成功创建。

        # 定义API请求的URL。这个URL用于向指定的线程中运行指定的助手。
        url = f"{self.MINMAX_HOST}/threads/run/create?GroupId={self.group_id}"

        # 定义请求的负载数据。这个数据包含了要运行的线程ID和助手ID。
        payload = json.dumps(
            {
                "thread_id": thread_id,  # 线程的唯一标识符。助手将在这个线程中运行。
                "assistant_id": assistant_id,  # 助手的唯一标识符。指定了要运行的助手。
            }
        )

        # 发送POST请求到API，传递请求头和负载数据。
        response = requests.post(url, headers=self.header(), data=payload)

        # 返回API的响应结果，以JSON格式返回。
        return response.json()

    # 查看运行状态
    def check_thread_run_status(self, thread_id, run_id):
        # 定义请求的URL，构造获取线程运行状态的API端点。
        # 这里URL包括了`GroupId`作为查询参数，表示请求的是特定群组中的运行状态。
        url = f"{self.MINMAX_HOST}/threads/run/retrieve?GroupId={self.group_id}"

        # 创建请求的负载数据，包含线程ID和运行ID。
        # 这些数据将用于API请求中，以指定要查询的线程和运行状态。
        payload = json.dumps(
            {
                "thread_id": str(thread_id),  # 线程ID，指定要查询的线程
                "run_id": str(run_id),  # 运行ID，指定要查询的具体运行
            }
        )

        # 初始化`completed`变量为`False`，表示任务尚未完成。
        # 当我们把这个变量修改成True,那么表示任务完成
        completed = False

        # 使用循环不断检查任务的运行状态，直到任务完成。
        while not completed:
            # 发送GET请求到指定的URL，附带请求头和负载数据。
            response = requests.request("GET", url, headers=self.header(), data=payload)

            # 检查响应状态码是否为200，表示请求成功。
            if response.status_code == 200:
                # 将API响应的内容转换为JSON格式。
                response_data = response.json()

                # 从响应数据中提取`status`字段，获取任务的当前状态。
                status = response_data.get("status", "")

                # 打印当前任务状态。
                print(f"Status: {status}")

                # 检查任务状态是否为`completed`，如果是，标记为完成。
                if status == "completed":
                    completed = True
                    print("Process completed, exiting loop.")
                else:
                    # 如果状态不是`completed`，等待两秒后重新请求。
                    time.sleep(2)
            else:
                # 如果请求失败，打印错误信息，并退出循环。
                print(f"Error: {response.status_code}")
                break

        # 返回任务是否完成的状态。
        return completed

    def get_thread_messages(self, thread_id):
        # 定义要请求的URL，使用线程ID来构造URL，以便获取该线程的消息列表
        url = f"{self.MINMAX_HOST}/threads/messages/list?GroupId={self.group_id}"

        # 创建请求的负载数据，包含线程ID
        payload = json.dumps({"thread_id": thread_id})  # 包含要查询消息的线程ID

        # 发送GET请求到指定的URL，附带请求头和负载数据
        response = requests.get(url, headers=self.header(), data=payload)

        # 将响应内容转换为JSON格式并返回
        return response.json()

    def answer(self) -> string:
        # 上传文档
        file_response = self.create_file()
        file_id = file_response.get("file", {}).get("file_id")
        if is_string_empty(file_id):
            print("上传知识库失败")
            return

        print("上传知识库完成 fileId==", file_id)

        # 创建助手
        assistant_response = self.create_assistant(file_id)
        # print(assistant_response)
        print("****************")
        assistant_id = assistant_response.get("id", "")
        if is_string_empty(assistant_id):
            print("创建助手失败")
            return

        print("创建助手完成 assitantId=", assistant_id)

        # 创建线程
        thread_response = self.create_thread()
        # print(thread_response)
        print("***************")
        thread_id = thread_response.get("id", "")
        if is_string_empty(assistant_id):
            print("创建线程失败")
            return
        print("创建线程完成 threadId=", thread_id)

        self.add_message_to_thread(thread_id, self.prompt)

        # 开始运行线程
        run_response = self.run_thread_with_assistant(
            thread_id=thread_id,
            assistant_id=assistant_id,
        )
        print("线程运行情况\n", run_response)
        run_id = run_response.get("id", "")
        # 假设run_response是正确的JSON响应，并包含run_id
        print("run_id:", run_id)

        # 检查助手处理状态
        if self.check_thread_run_status(thread_id, run_id):
            # 获取线程中助手处理出的新信息
            thread_messages_response = self.get_thread_messages(thread_id)
            # print(type(thread_messages_response))
            return thread_messages_response["data"][1]["content"][0]["text"]["value"]


if __name__ == "__main__":
    answer = MyAssitantAi(
        group_id="1820662545770365832",
        api_key="eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJHcm91cE5hbWUiOiLlkajoiKoiLCJVc2VyTmFtZSI6IuWRqOiIqiIsIkFjY291bnQiOiIiLCJTdWJqZWN0SUQiOiIxODIwNjYyNTQ1Nzc4NzU0NDQwIiwiUGhvbmUiOiIxMzIxNzMwMDIzNyIsIkdyb3VwSUQiOiIxODIwNjYyNTQ1NzcwMzY1ODMyIiwiUGFnZU5hbWUiOiIiLCJNYWlsIjoiIiwiQ3JlYXRlVGltZSI6IjIwMjQtMDktMjIgMTA6MDc6MTkiLCJpc3MiOiJtaW5pbWF4In0.nDjc9FYKBgbXcGIRoOE7NG7xTjPpvZbg8c7ZaoMJb2g556l2yh1Dcirms71G8Te0girecGT4Wtd21GCtq3Cm6nEUGgEXVCBHa3bLIiLgsySToFZuosvjti7GZ3ojqYBn7y75qdFUPbg4oOqtu5QD_FHPaiLHqCm9yD32qUYYJLVygD0a95JXNthlQGKF6aPKAfN2oy9aOPmNViMi7roZhe6tYSL8SKLuW1uWEp7jYN5KkyaNwnGzrM4aWXUOGcuCslpqa4HM20kk1LeN0xFihn88XXFIW3uZnZJ0YRn8oJanpc6EgdK5edP7KqydW3QaBwPzfe_HLuKlQDwd_Wol6g",
        file_path="显卡价目表.md",
        prompt="假如我有3500块,帮我选一款性价比尽可能高的显卡,并给出详细理由",
    ).answer()

    print("AI输出的结果是\n", answer)
